// -----------------------------------------------------------------------------
// (C) Bibix
// The content below includes confidential, proprietary information of
// Bibix. All use, disclosure, and/or reproduction is prohibited
// unless authorized in writing. All rights reserved.
// -----------------------------------------------------------------------------
// File         : tb_movaver.cpp
// Description  : SystemC testbench for moving-average filter
// Author       : Sabih Gerez, Bibix
// Creation date: August 18, 2010
// -----------------------------------------------------------------------------
// $Rev: 47 $
// $Author: sabih $
// $Date: 2011-02-03 01:25:20 +0100 (Thu, 03 Feb 2011) $
// $Log$
// -----------------------------------------------------------------------------

#include "tb_movaver.h"

// necessary for Questasim interface
SC_MODULE_EXPORT(tb_movaver);

// member functions

// *** From Questasim example set ***
// Reset pulse generator.
// The first time it runs at initialization and sets reset low.
// It schedules a wakeup at time 400 ns, and at that time sets
// reset high (inactive).

inline void tb_movaver::reset_generator()
{
    static bool first = true;
    if (first)
    {
        first = false;
        resetn.write(SC_LOGIC_0);
        reset_deactivation_event.notify(400, SC_NS);
    }
    else
        resetn.write(SC_LOGIC_1);
}

// from clock object to clock signal
inline void tb_movaver::clock_assign()
{
    sc_logic clock_tmp(clock.signal().read());
    iclock.write(clock_tmp);
}

// prologue: open input and output files
void tb_movaver::tb_prologue()
{
  infile = fopen("movaver.in", "rt");
  if (infile == NULL)
     printf("Cannot open movaver.in for reading!\n");
  assert(infile != NULL);

  outfile = fopen("movaver.out", "wt");
  if (outfile == NULL)
     printf("Cannot open movaver.out for writing!\n");
  assert(outfile != NULL);
}

// epilogue: close input and output files and terminate simulation
void tb_movaver::tb_epilogue()
{
  fclose(infile);
  fclose(outfile);
  printf("OK! End of input file reached. Stopping simulation.\n");
  sc_stop();
}

// test-vector controller: testbench actions to be repeated at each
// clock cycle
void tb_movaver::tvc_movaver()
{
  // read one line from input file
  // no more than 10 characters per line in input file
  char line[10];
  int in_number;
  int out_data;

  // static variable on behalf of a periodic clear
  static int input_counter = 0;
  
  if (resetn.read() == '1') {
     if (fgets(line, 10, infile) != NULL)  {
        // calculate periodic clear
        int local_clear;
        if (input_counter++ == 10) {
           input_counter = 0;
           local_clear = 1;
        }
        else
           local_clear = 0;
      
        sscanf(line, "%d", &in_number);
        data_in.write(in_number);
        // drive clear
        clear.write(sc_logic(local_clear));
        out_data = (data_out.read()).to_int();
        fprintf(outfile, "%6d %6d\n", in_number, (int)out_data);
     }
     else {
        // end of file -> end of simulation
        eos_event.notify(1, SC_NS);
     }
  } 
  else {
     // drive DUV in reset
     clear.write(sc_logic('0'));
     data_in.write(10);
  }
}
