// -----------------------------------------------------------------------------
// (C) Bibix
// The content below includes confidential, proprietary information of
// Bibix. All use, disclosure, and/or reproduction is prohibited
// unless authorized in writing. All rights reserved.
// -----------------------------------------------------------------------------
// File         : tb_movaver.h
// Description  : SystemC testbench for moving-average circuit, header file
// Author       : Sabih Gerez, Bibix
// Creation date: August 17, 2010
// -----------------------------------------------------------------------------
// $Rev: 47 $
// $Author: sabih $
// $Date: 2011-02-03 01:25:20 +0100 (Thu, 03 Feb 2011) $
// $Log$
// -----------------------------------------------------------------------------

#ifndef INCLUDED_TB_MOVAVER
#define INCLUDED_TB_MOVAVER

#include <systemc.h>
#include "movaver.h"

// "generics"
#define WL 10

SC_MODULE(tb_movaver)
{
    // clock
    sc_clock clock;

    // used for reset
    sc_event reset_deactivation_event;

    // end of simulation event
    sc_event eos_event;

    // local signals
    sc_signal<sc_logic> iclock;

    sc_signal<sc_logic> resetn;
    sc_signal<sc_logic> clear;
    sc_signal<sc_lv<WL> > data_in;
    sc_signal<sc_lv<WL> > data_out;

    // pointer to filter object
    movaver_std* movaver_INST;

    // file pointers used in testbench
    FILE *infile;
    FILE *outfile;

    // methods
    void reset_generator();
    void clock_assign();
    void tvc_movaver();
    void tb_prologue();
    void tb_epilogue();

    // constructor for testbench object
    SC_CTOR(tb_movaver): 
      clock("clock", 200, SC_NS, 0.5, 0.0, SC_NS, false),
      iclock("iclock"),
      resetn("resetn"),
      clear("clear"),
      data_in("data_in"),
      data_out("data_out")
    {
      // create instance of filter object
      movaver_INST = new movaver_std ("movaver_INST", "work.movaver_std");

      // wire filter
      movaver_INST->clock(iclock);
      movaver_INST->resetn(resetn);
      movaver_INST->clear(clear);
      movaver_INST->data_in(data_in);
      movaver_INST->data_out(data_out);

      // reset
      SC_METHOD(reset_generator);
      sensitive << reset_deactivation_event;

      // clock
      SC_METHOD(clock_assign);
      sensitive << clock.signal();
      dont_initialize();

      // test-vector controller
      SC_METHOD(tvc_movaver);
      sensitive << clock.negedge_event();

      // initialize testbench; should only run once; no sensitivity
      SC_METHOD(tb_prologue);

      // epilogue; post-process results and stop simulation
      SC_METHOD(tb_epilogue);
      dont_initialize();
      sensitive << eos_event;
    }

    // destructor
    ~tb_movaver()
    {
      delete movaver_INST;
      movaver_INST = 0;
    }
// note: semicolon after class defintion is essential!
};

#endif
